transfer_api_port = 7003
transfer_api_secured = False
resource_service_host = "localhost"
resource_service_port = 7003
resource_service_secured = False
secret_service_host = "localhost"
secret_service_port = 7003